--[[
  Orbit mechanics calculating script for use with the Orbit-anim.mpd LDCadVEX example.

  Philippe 'Philo' Hurbain, with many code portions borrowed from Roland Melkert!
]]


function register()

  --[[
    First thing to do in a LDCad animation script is telling the LDCad internals we are here.
    It's best to this in a function using local variables.
    This because a script's is called during loading of models and or when script changes were detected.
    Using global variables in such an environment will cause them to live for the entirely life time of the script.
    Which might be confusing when using multiple animations and or similar named variables.
  ]]

  --First register the new animation so it becomes available in the GUI
  -- This is done using an animation object which will be our lua interface to the animations internals.
  local ani=ldc.animation('Orbit')
  --Number of similated days
  days=365
  --Duration of a simulated day in seconds
  local duration = 0.2
  --Set the length of the animation in seconds
  ani:setLength(days*duration)

  --Link a lua function to be called whenever the animation is about to be used.
  -- This event is called only once when an animation goes from inactive to active.
  -- The start event can be used to buffer some time intensive calculations in order to minimize the per frame calculations.
  ani:setEvent('start', 'onStart')

  --Link a lua function to be called whenever LDCad is about to render a different frame of our animation.
  -- The main goal of a frame event handler is to apply the state associated to a specific point in time to the entire model.
  -- This means you must set the absolute desired position/orientation/color and visibility of all non static objects in your animation.
  -- This is usually done based upon the current time.
  ani:setEvent('frame', 'onFrame')
end

function onStart()

  --Setup a collection of interfaces to stuff we want to animate.
  -- This is done using global variables in order to also access then from the onFrame function.

  --The top level LDraw file.
  	mainSf=ldc.subfile()
	--Get reference to crank various axle subfiles
	crank=mainSf:getRef('crank.ldr')
	axle2=mainSf:getRef('axle2.ldr')
	axle3=mainSf:getRef('axle3.ldr')
	axle4=mainSf:getRef('axle4.ldr')
	axle5=mainSf:getRef('axle5.ldr')
	ring=mainSf:getRef('ring.ldr')
	axleb1=mainSf:getRef('axleb1.ldr')
	axleb2=mainSf:getRef('axleb2.ldr')
	axleb3=mainSf:getRef('axleb3.ldr')
	axleb4=mainSf:getRef('axleb4.ldr')
	axleb5=mainSf:getRef('axleb5.ldr')
	axlec1=mainSf:getRef('axlec1.ldr')
	axlec2=mainSf:getRef('axlec2.ldr')
	axlec3=mainSf:getRef('axlec3.ldr')
	armbase=mainSf:getRef('armbase.ldr')
	--Get access to the arm subfiles
	armbaseSf=ldc.subfile('armbase.ldr')
	--And to gear axles depending on it.	
	axlee1=armbaseSf:getRef('axlee1.ldr')
	axlee2=armbaseSf:getRef('axlee2.ldr')
	axlee3=armbaseSf:getRef('axlee3.ldr')
	axlem1=armbaseSf:getRef('axlem1.ldr')
	axlem2=armbaseSf:getRef('axlem2.ldr')
	axlem3=armbaseSf:getRef('axlem3.ldr')
end

function onFrame()
  --Get an interface to the active animation.
  local ani=ldc.animation.getCurrent()

  --Buffer some animation state info
  local curTime=ani:getFrameTime()
  local totTime=ani:getLength()
  --Turn crank CW
  local crankAngle=-curTime/totTime*360*days
  local ori=ldc.matrix()
  ori:setRotate(crankAngle+180, 1, 0, 0)
  crank:setOri(ori)
  ori:setRotate(-crankAngle, 1, 0, 0)
  axle2:setOri(ori)
  axleb1:setOri(ori)
  ori:setRotate(crankAngle, 0, 1, 0)
  axle3:setOri(ori)
  axle5:setOri(ori)
  axlec1:setOri(ori)
  ori:setRotate(-crankAngle, 0, 1, 0)
  axle4:setOri(ori)
  local ringspeed=crankAngle*12/36
  ori:setRotate(ringspeed, 0, 1, 0)
  ring:setOri(ori)
  local yearspeed=crankAngle*4/195
  ori:setRotate(-yearspeed, 0, 1, 0)
  axleb2:setOri(ori)
  ori:setRotate(yearspeed*24/36, 0, 1, 0)
  axleb3:setOri(ori)
  axleb5:setOri(ori)
  ori:setRotate(-yearspeed*24/36, 0, 1, 0)
  axleb4:setOri(ori)
  ori:setRotate(-crankAngle*15/36, 0, 1, 0)
  axlec2:setOri(ori)  
  ori:setRotate(crankAngle*15/36*12/48, 0, 1, 0)
  axlec3:setOri(ori)
  local armspeed=yearspeed*24/36*12/60+180
  ori:setRotate(armspeed, 0, 1, 0)
  armbase:setOri(ori)
  local earthspeed=(ringspeed-armspeed)*36/12
  ori:setRotate(earthspeed, 0, 1, 0)
  axlee1:setOri(ori)
  axlee3:setOri(ori)  
  ori:setRotate(earthspeed, 1, 0 , 0)
  axlee2:setOri(ori)
  local moonspeed=-crankAngle*15/36*12/48+armspeed
  ori:setRotate(moonspeed-75, 1, 0 , 0)
  axlem1:setOri(ori)
  ori:setRotate(moonspeed+5, 0, 1 , 0)
  axlem2:setOri(ori)
  ori:setRotate(-moonspeed*12/36+117, 0, 1 , 0)
  axlem3:setOri(ori)

end

--Take care of adding the animation to the LDCad gui etc.
register()
